@@ -13,12 +13,26 @@ use super::{Number, _Complex};
 pub trait NumberClass: Default + Into<NumberType> + Ord + Send + fmt::Display {
     type Instance: NumberInstance;
 
+    /// Cast the given `Number` into an instance of this type.
     fn cast(&self, n: Number) -> Self::Instance;
 
+    /// Return `true` if this is a complex type.
+    fn is_complex(&self) -> bool {
+        return false;
+    }
+
+    /// Return `false` if this is a complex type.
+    fn is_real(&self) -> bool {
+        !self.is_complex()
+    }
+
+    /// Return the maximum size of this type of [`Number`], in bits.
     fn size(self) -> usize;
 
+    /// Return `1` as an instance of this type.
     fn one(&self) -> <Self as NumberClass>::Instance;
 
+    /// Return `0` as an instance of this type.
     fn zero(&self) -> <Self as NumberClass>::Instance;
 }
 
@@ -58,8 +72,13 @@ pub trait NumberInstance:
     /// Calculate the absolute value of this number.
     fn abs(self) -> Self::Abs;
 
+    /// Raise `e` to the power of this number.
+    fn exp(self) -> Self::Exp;
+
     /// Raise this number to the given exponent.
-    fn pow(self, exp: Self::Exp) -> Self;
+    ///
+    /// Panics: if the given exponent is a complex number.
+    fn pow(self, exp: Number) -> Self;
 
     /// Return `true` if `self` and `other` are nonzero.
     fn and(self, other: Self) -> Self
@@ -133,6 +152,10 @@ impl NumberClass for ComplexType {
         }
     }
 
+    fn is_complex(&self) -> bool {
+        true
+    }
+
     fn size(self) -> usize {
         match self {
             Self::C32 => 8,
